/*  libadfs - library to manipulate old BBC Micro/Master ADFS images
 *  Copyright (C) 2008  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *  */

#include "config.h"
#include "adfs.h"
#include "debug.h"
#include "dir.h"
#include "disk.h"

#include <stdio.h>
#include <string.h>

static const char *boot_option_string[4] = {
	"Off",
	"Load",
	"Run",
	"Exec"
};

/* Format-specific extended directory listing */
int ADFS_extended_ls(struct ADFS_disk *disk, const char *path) {
	ADFS_DIR *dir;
	struct ADFS_dirent *ent;
	int boot_option;
	char opts[5];

	dir = ADFS_opendir(disk, path);
	if (dir == NULL)
		return -1;

	dir->index = 0;  /* skip . and .. */
	boot_option = disk->boot_option;
	printf("%-20s(%02X)\n", dir->title, dir->cycle_number);
	printf("Drive:0             Option %02X (%-4s)\n", boot_option, boot_option_string[boot_option]);
	printf("Dir. %-15sLib. \"Unset\"\n\n", dir->name);
	while ((ent = ADFS_readdir(dir))) {
		opts[0] = 0;
		if (ent->attr & ADFS_ATTR_DIR) strcat(opts, "D");
		if (ent->attr & ADFS_ATTR_LOCKED) strcat(opts, "L");
		if (ent->attr & ADFS_ATTR_WRITE) strcat(opts, "W");
		if (ent->attr & ADFS_ATTR_READ) strcat(opts, "R");
		if (ent->attr & ADFS_ATTR_DIR) {
			printf("%-10s %-3s(%02X)  %06X\n", ent->name, opts, ent->cycle_number, ent->start_lsn);
		} else {
			printf("%-10s %-3s(%02X)  %08X  %08X  %08X  %06X\n", ent->name, opts, ent->cycle_number, ent->load_address, ent->exec_address, ent->length, ent->start_lsn);
		}
	}
	ADFS_closedir(dir);
	return 0;
}
