/*  libadfs - library to manipulate old BBC Micro/Master ADFS images
 *  Copyright (C) 2008  Ciaran Anscomb
 *
 *  See COPYING.GPL for redistribution conditions. */

#ifndef _ADFS_H_
#define _ADFS_H_

#include <sys/types.h>

struct ADFS_disk;
struct ADFS_FILE;
typedef struct ADFS_FILE ADFS_FILE;

#define ADFS_ATTR_READ   (1<<0)
#define ADFS_ATTR_WRITE  (1<<1)
#define ADFS_ATTR_LOCKED (1<<2)
#define ADFS_ATTR_DIR    (1<<3)
#define ADFS_ATTR_EXEC   (1<<4)

struct ADFS_dirent {
	char name[11];
	int attr;
	unsigned int load_address;
	unsigned int exec_address;
	unsigned int length;
	int start_lsn;
	int cycle_number;
	/* private: */
	struct ADFS_disk *disk;
	int parent_dir_lsn;
	int parent_dir_index;
	int modified;
};

struct ADFS_DIR {
	char name[11];
	char title[20];
	int cycle_number;
	/* private: */
	ADFS_FILE *fd;
	struct ADFS_dirent ent_buf;
	int index;
};
typedef struct ADFS_DIR ADFS_DIR;

struct ADFS_disk *ADFS_open_disk(const char *filename, int oflag);
void ADFS_close_disk(struct ADFS_disk *disk);

int ADFS_get_boot_option(struct ADFS_disk *disk);
void ADFS_set_boot_option(struct ADFS_disk *disk, int option);

ADFS_FILE *ADFS_creat(struct ADFS_disk *disk, const char *path, int mode);
ADFS_FILE *ADFS_open(struct ADFS_disk *disk, const char *path, int oflag, int mode);
int ADFS_close(ADFS_FILE *fd);

int ADFS_fsync(ADFS_FILE *fd);
int ADFS_ftruncate(ADFS_FILE *fd, off_t length);
off_t ADFS_lseek(ADFS_FILE *fd, off_t offset, int whence);
ssize_t ADFS_read(ADFS_FILE *fd, void *buf, size_t nbytes);
ssize_t ADFS_write(ADFS_FILE *fd, const void *buf, size_t nbytes);

int ADFS_fstat(ADFS_FILE *fd, struct ADFS_dirent *buf);
int ADFS_set_attribute(ADFS_FILE *fd, const char *attr, const char *value);

ADFS_DIR *ADFS_opendir(struct ADFS_disk *disk, const char *path);
void ADFS_closedir(ADFS_DIR *dir);
int ADFS_readdir_r(ADFS_DIR *dir, struct ADFS_dirent *entry,
                   struct ADFS_dirent **result);
struct ADFS_dirent *ADFS_readdir(ADFS_DIR *dir);
void ADFS_rewinddir(ADFS_DIR *dirp);
int ADFS_mkdir(struct ADFS_disk *disk, const char *path);
int ADFS_chdir(struct ADFS_disk *disk, const char *path);
int ADFS_fchdir(ADFS_FILE *fd);

int ADFS_rename(struct ADFS_disk *disk, const char *old, const char *new);
int ADFS_remove(struct ADFS_disk *disk, const char *path);

int ADFS_extended_ls(struct ADFS_disk *disk, const char *path);

#endif /* _ADFS_H_ */
