/*  libadfs - library to manipulate old BBC Micro/Master ADFS images
 *  Copyright (C) 2008  Ciaran Anscomb
 *
 *  See COPYING.GPL for redistribution conditions. */

#ifndef _ADFS_DISK_H_
#define _ADFS_DISK_H_

#include <stdint.h>

#define SECTOR_SIZE (256)
#define DIRECTORY_SECTORS (5)
#define MAX_FSM_ENTRIES (82)

#define DIRECTORY_LENGTH (SECTOR_SIZE * DIRECTORY_SECTORS)

struct fsm_entry {
	int lsn;
	int size;	/* in sectors */
};

struct ADFS_disk {
	int fd;  /* of file containing disk image */
	int sparse;  /* 1 = normal, 2 = sparse (e.g. IDE/CF image) */
	int translate;  /* 1 = disk not in logical sector order */
	int num_fsm_entries;
	int max_fsm_entries;
	struct fsm_entry fsm[MAX_FSM_ENTRIES];
	int total_sectors;
	int disk_id;
	int boot_option;
	struct ADFS_dirent root_dirent;
	struct ADFS_dirent *cwd_dirent;
	int modified;  /* flag indicating need to rewrite FSM */
};

int read_sector(struct ADFS_disk *disk, int lsn, uint8_t *buf); 
int write_sector(struct ADFS_disk *disk, int lsn, uint8_t *buf);
void coalesce_fsm(struct ADFS_disk *disk);
int rewrite_fsm(struct ADFS_disk *disk);

#endif  /* _ADFS_DISK_H_ */
